from boutdata.collect import collect
from boututils.moment_xyzt import moment_xyzt
from boututils.file_import import file_import
from pylab import save, figure, plot, title, xlabel, ylabel, show, tight_layout
from elm_size import elm_size

path='./data'

t_array=collect('t_array', path=path)
save('t_array.dat', t_array)
p0=collect('P0', path=path)
save('p0.dat', p0)


# n0=collect('n0', path=path)
# save('n0.dat', n0
# ti0=collect('ti0', path=path)
# save('ti0.dat', ti0)
# te0=collect('te0', path=path)
# save('te0.idl.dat', te0)

gfile=file_import('./cbm18_dens8.grid_nx68ny64.nc')

p=collect('P', path=path)
save('p.dat', p)
res=moment_xyzt(p,'RMS','DC')
rmsp=res.rms
dcp=res.dc
save('rmsp.dat', rmsp)
save('dcp.dat',  dcp)
elmsp=elm_size(dcp,p0,gfile,yind=32,Bbar=gfile['bmag'])
save('elmsp.dat',  elmsp)

figure(0)
plot(t_array,elmsp.s2, 'k-')
xlabel('t/Ta')
ylabel('Elm size')
title('Elm size, P')
tight_layout()
show()


phi=collect('phi', path=path )
save('phi.dat', phi)
res=moment_xyzt( phi, 'DC', 'RMS')
save('dcphi.dat',res.dc)
save('rmsphi.dat', res.rms)



from __future__ import print_function
from __future__ import division
from builtins import str
from builtins import range
from past.utils import old_div

from numpy import *;
#from scipy.io import readsav;
import matplotlib.pyplot as plt;

# Dynamic matplotlib settings
from matplotlib import rcParams;
rcParams['font.size'] = 20;
rcParams['legend.fontsize'] = 'small';
rcParams['legend.labelspacing'] = 0.1;
rcParams['lines.linewidth'] = 2;
rcParams['savefig.bbox'] = 'tight';

# Create image directory if not exists
import os;
if not os.path.exists('image'):
   os.makedirs('image');

#fphi = transpose(readsav('fphi.idl.dat')['fphi'])[:,:,:,];
fphi = load('fp.npy')

plt.figure();
for i in range(1, 9):
   print("Growth rate for mode number", i)
   print(gradient(log(abs(fphi[34, 32, i, :]))))
   plt.semilogy(((abs(fphi[34, 32, i, :]))), label = 'n=' + str(i * 5));

plt.legend(loc=2);
plt.xlabel('Time');
plt.savefig('image/plotmode.png');
plt.savefig('image/plotmode.eps');


plt.show(block=False);
plt.figure();
for i in range(1, 9):
   plt.plot(abs(fphi[:, 32, i, -1]), label = 'n=' + str(i * 5));

plt.legend();
plt.xlabel('X index');

plt.savefig('image/plotmodeamp.png');
plt.savefig('image/plotmodeamp.eps');

plt.show(block=False);

plt.figure();
for i in range(1, 9):
   plt.plot(old_div(abs(fphi[:, 32, i, -1]),abs(fphi[:, 32, i, -1]).max()), label = 'n=' + str(i * 5));

plt.legend();
plt.xlabel('X index');

plt.savefig('image/plotmodenorm.png');
plt.savefig('image/plotmodenorm.eps');

plt.show();




import numpy as np
from boutdata.collect import collect
import pickle

try:
        from past.utils import old_div
except ImportError:
        def old_div(a,b):
                return a/b


def blob_velocity(n,**kwargs):

        from boututils import calculus as Calc
        # Calculate blob velocity in normalized time and normalized grid spacing
        #
        # Input: Blob density as a 3D vector in the form  n[t,x,z] where t is time and x,z are the perpendicular spatial coordinates
        #
        # Keywords:
        #
        #           type='peak' -> Calculate velocity of the peak density
        #           type='COM' -> Calculate centre of mass velocity
        #           Index=True -> return indices used to create velocity
        #
        # Default: Peak velocity with no index returning

        size = n.shape

        try:
                v_type = kwargs['type']
        except:
                v_type = 'peak'         #Default to peak velocity calculation
        try:
                return_index = kwargs['Index']
        except:
                return_index = False    #Default to no index returning


        if v_type == 'peak':
                x = np.zeros(size[0])
                z = np.zeros(size[0])
                for i in np.arange(size[0]):
                        nmax,nmin = np.amax((n[i,:,:])),np.amin((n[i,:,:]))
                        xpos,zpos = np.where(n[i,:,:]==nmax)
                        x[i] = xpos[0]
                        z[i] = zpos[0]

        if v_type == 'COM':
                x = np.zeros(size[0])
                z = np.zeros(size[0])
                for i in np.arange(size[0]):
                        data = n[i,:,:] - n[0,0,0]   #use corner cell rather than nmin
                        ntot = np.sum(data[:,:])

                        z[i] = old_div(np.sum(np.sum(data[:,:],axis=0)*(np.arange(size[2]))),ntot)
                        x[i] = old_div(np.sum(np.sum(data[:,:],axis=1)*(np.arange(size[1]))),ntot)

        vx = Calc.deriv(x)
        vz = Calc.deriv(z)

        if return_index:
                return vx,vz,x,z
        else:
                return vx,vz



data='data'

if True:
        import sys
        if len(sys.argv) > 1:
                data=sys.argv[1]

n = collect('n', path=data, info=False)


vx,vy,xx,yy = blob_velocity(n[:,:,0,:],type='COM',Index=True)

f = open('Velocity.dat','wb')
pickle.dump(vx,f)
f.close()

f = open('Position.dat','wb')
pickle.dump(xx,f)
f.close()


f = open('Velocity.dat','rb')
vx = pickle.load(f)
f.close()

try:
        import matplotlib.pyplot as plt
        plt.plot(vx)
        plt.show()
except ImportError:
        pass



from __future__ import print_function
from __future__ import division
from builtins import range
from past.utils import old_div
import numpy as np

def elm_size(dcp,p0,uedge,xmin=None,xmax=None,yind=None,Bbar=None):
  
    lis=[dcp,p0,uedge]
    if np.size(lis) != 3 :
        print("lack of parameters")
        return 0
    

    if xmin == None : xmin=0
    if xmax == None : xmax=327 
    if yind == None : yind=63 # choose the poloidal location for 1D size
    if Bbar == None : Bbar=1.992782  # the normalized magnetic field 

    mydcp=dcp
    myp0=p0
    g=uedge

    PI = 3.1415926
    MU0 = 4.0e-7*PI

    s=np.shape(mydcp)

    if np.ndim(mydcp) != 3 :
        print("dcp should be 3D(t,x,y)")
 

    nt=s[0]
    nx=s[1]
    ny=s[2]

    Dtheta=g['dy']     #using correct poloidal angle
    psixy=g['psixy']
    R=g['Rxy']
    Bp=g['Bpxy']
    hthe=g['hthe']

    Dpsi=np.zeros((nx,ny))
    Dpsi[0,:]=psixy[1,:]-psixy[0,:]
    Dpsi[nx-1,:]=psixy[nx-1,:]-psixy[nx-2,:]
    for i in range(1,nx-2):
        Dpsi[i,:]=old_div((psixy[i+1,:]-psixy[i-1,:]),2)
     

    Ddcp1=np.zeros(nt)
    Ddcp2=np.zeros(nt)
    Ddcp3=np.zeros(nt)
    Tp01=0.
    Tp02=0.
    Tp03=0.

    for t in range(nt) :
        Ddcp3[t]=2.0*PI*np.sum(mydcp[t,xmin:xmax,:]*hthe[xmin:xmax,:]*Dtheta[xmin:xmax,:]*Dpsi[xmin:xmax,:]/Bp[xmin:xmax,:])
        Ddcp2[t]=np.sum(mydcp[t,xmin:xmax,:]*hthe[xmin:xmax,:]*Dtheta[xmin:xmax,:]*Dpsi[xmin:xmax,:]/(R[xmin:xmax,:]*Bp[xmin:xmax,:]))
        Ddcp1[t]=np.sum(mydcp[t,xmin:xmax,yind]*Dpsi[xmin:xmax,yind]/(R[xmin:xmax,yind]*Bp[xmin:xmax,yind])) 
    

    Tp03=2.0*PI*np.sum(myp0[xmin:xmax,:]*hthe[xmin:xmax,:]*Dtheta[xmin:xmax,:]*Dpsi[xmin:xmax,:]/Bp[xmin:xmax,:])
    Tp02=np.sum(myp0[xmin:xmax,:]*hthe[xmin:xmax,:]*Dtheta[xmin:xmax,:]*Dpsi[xmin:xmax,:]/(R[xmin:xmax,:]*Bp[xmin:xmax,:]))
    Tp01=np.sum(myp0[xmin:xmax,yind]*Dpsi[xmin:xmax,yind]/(R[xmin:xmax,yind]*Bp[xmin:xmax,yind]))

    s1=np.zeros(nt)
    s2=np.zeros(nt)
    s3=np.zeros(nt)
    E_loss=np.zeros(nt)

    s1=old_div(-Ddcp1,Tp01)   #1D elm size
    s2=old_div(-Ddcp2,Tp02)   #2D elm size
    s3=old_div(-Ddcp3,Tp03)   #3D elm size

    E_loss=-Ddcp3*(0.5*Bbar*Bbar/MU0)    #energy loss, unit J
    E_total=Tp03*(0.5*Bbar*Bbar/MU0)     #total energy, unit J

    class ELM:
        pass
    elmsize=ELM()
    elmsize.s1=s1
    elmsize.s2=s2
    elmsize.s3=s3
    elmsize.E_loss=E_loss
    elmsize.E_total=E_total

    return elmsize
  
    



#!/usr/bin/env python

from boutdata import restart
from sys import argv, exit

npes = int(argv[1])

try:
    restarts_directory = argv[2]
except IndexError:
    restarts_directory = "restarts_256x256"

restart.redistribute(npes, path=restarts_directory, output="data", myg=0)

exit(0)



#!/usr/bin/env python3
from __future__ import division
from past.utils import old_div

import matplotlib.pyplot as plt
import numpy as np
from boututils.moment_xyzt import moment_xyzt
from boututils.file_import import file_import
from boutdata.collect import collect
import os

#Dynamic matplotlib settings
from matplotlib import rcParams
rcParams['font.size'] = 20.
rcParams['legend.fontsize'] = 'small'
rcParams['lines.linewidth'] = 2

if not os.path.exists('image'):
   os.makedirs('image')

g = file_import(os.path.dirname(os.path.realpath(__file__))+'/../cbm18_dens8.grid_nx68ny64.nc')
psi = old_div((g['psixy'][:, 32] - g['psi_axis']), (g['psi_bndry'] - g['psi_axis']))

path = './data'

plt.figure()

p0=collect('P0', path=path)

p=collect('P', path=path)
res = moment_xyzt(p,'RMS','DC')
rmsp = res.rms
dcp = res.dc
nt = dcp.shape[0]

plt.plot(psi, p0[:, 32], 'k--', label='t=0')
plt.plot(psi, p0[:, 32] + dcp[nt//4, :, 32], 'r-', label='t='+np.str(nt//4))
plt.plot(psi, p0[:, 32] + dcp[nt//2, :, 32], 'g-', label='t='+np.str(nt//2))
plt.plot(psi, p0[:, 32] + dcp[3*nt//4, :, 32], 'b-', label='t='+np.str(3*nt//4))
plt.plot(psi, p0[:, 32] + dcp[-1, :, 32], 'c-', label='t='+np.str(nt))

plt.legend()
#plt.xlim(0.6, 1.0)
plt.xlabel(r'Normalized poloidal flux ($\psi$)')
plt.ylabel(r'$\langle p\rangle_\xi$')
plt.title(r'Pressure')
xmin, xmax = plt.xlim()
ymin, ymax = plt.ylim()


#plt.savefig('image/plotcollapse.png', bbox_inches='tight')
#plt.savefig('image/plotcollapse.eps', bbox_inches='tight')


plt.tight_layout()

print("Showing plot")
plt.show()



from __future__ import print_function
from boututils.file_import import file_import
from boututils.volume_integral import volume_integral
from boutdata.collect import collect
# Integrate over a volume


path='./data/'

gfile='./cbm18_dens8.grid_nx68ny64.nc'

g = file_import(gfile)

var=collect("P", path=path)

sol=volume_integral(var, g)

print(sol)



from __future__ import division
from builtins import range
from past.utils import old_div
import numpy as np
import matplotlib.pyplot as plt
from boututils.file_import import file_import
from matplotlib.ticker import FixedFormatter, FormatStrFormatter, AutoLocator, AutoMinorLocator

g = file_import("./cbm18_dens8.grid_nx68ny64.nc")

majorLocator   = AutoLocator()
majorFormatter = FormatStrFormatter('%3.0e')
minorLocator   = AutoMinorLocator()
Fm = FixedFormatter(['0','$1 \\times 10^4$','$2 \\times  10^4$','$3 \\times 10^4$','$4 \\times 10^4$'])
Fm2 = FixedFormatter(['0','$2 \\times 10^5$','$4 \\times  10^5$','$6 \\times 10^5$'])

bxy=g.get('Bxy')
p=g.get('pressure')
jpar0=g.get('Jpar0')
psixy=g.get('psixy')
btxy=g.get('Btxy')
shiftangle=g.get('ShiftAngle')

nx=g.get('nx')
ny=g.get('ny')

q = np.zeros((nx, ny))
for i in range(ny):
    q[:,i] = old_div(- shiftangle, (2 * np.pi))




xarr = psixy[:,0]
xarr = old_div((xarr + 0.854856), (0.854856 + 0.0760856))


fig=plt.figure()
plt.plot(xarr,q[:,32])
plt.xlabel('normalized $\psi$', fontsize=25)
plt.ylabel('$q$',rotation='horizontal',fontsize=25)

fig.set_tight_layout(True)

fig, ax1 = plt.subplots()

ax1.plot(xarr, p[:,32], 'r-', markevery=1, linewidth=3)
ax1.set_xlabel('normalized $\psi$',fontsize=25)
# Make the y-axis label and tick labels match the line color.
ax1.set_ylabel('Pressure [Pa]', color='k',fontsize=25)

#set y limit
ax1.set_ylim(0,40000,10000)

#define ticks#
ax1.yaxis.set_ticks(np.arange(0, 40000, 10000))
#ax1.yaxis.set_major_locator(majorLocator)
#ax1.yaxis.set_major_formatter(majorFormatter)
ax1.yaxis.set_major_formatter(Fm)

#for the minor ticks, use no labels; default NullFormatter
ax1.xaxis.set_minor_locator(AutoMinorLocator())
ax1.yaxis.set_minor_locator(AutoMinorLocator(10))

#format tick labels
for tl in ax1.get_yticklabels():
    tl.set_color('k')


ax2 = ax1.twinx()

s2 = -jpar0

ax2.plot(xarr, s2[:,32], 'r-',markevery=1,linewidth=3)
ax2.set_ylabel('$J_\parallel  [A/m^2]$', color='k',fontsize=25)
ax2.set_ylim(0,600000)
ax2.yaxis.set_ticks(np.arange(0, 600000, 200000))
ax2.yaxis.set_major_formatter(Fm2)
for tl in ax2.get_yticklabels():
    tl.set_color('k')


fig.set_tight_layout(True)

plt.show()


#plt.savefig('2d.png', transparent=True)



from __future__ import print_function
from __future__ import division
from builtins import range
from past.utils import old_div
###
# computes average growth rate for all points at the final timestep
# computes average growth rate for points in the mead plane at the final timestep
###
import numpy as np
from boutdata.collect import collect
from boututils.moment_xyzt import moment_xyzt

path='./data/'

p=collect('P',path=path)

nmpy=old_div(p.shape[2],2)  # define mead plane

ik = 50 # disregard the first ik timesteps

def gr(p):
        rmsp_f=moment_xyzt(p, 'RMS').rms

        ni=np.shape(rmsp_f)[1]
        nj=np.shape(rmsp_f)[2]

        growth=np.zeros((ni,nj))

        for i in range(ni):
                for j in range(nj):
                         growth[i,j]=np.gradient(np.log(rmsp_f[ik::,i,j]))[-1]

        return growth


growth=gr(p)

d=np.ma.masked_array(growth,np.isnan(growth))

# masked arrays
# http://stackoverflow.com/questions/5480694/numpy-calculate-averages-with-nans-removed

print('Total mean value = ', np.mean(np.ma.masked_array(d,np.isinf(d))))
mm=np.ma.masked_array(growth[:,nmpy],np.isnan(growth[:,nmpy]))
if np.isinf(np.mean(mm)) :
    print('There is an Inf value in the mead plane')
    print('Mean value of floating numbers in mead plane is = ', np.mean(np.ma.masked_array(mm,np.isinf(mm))))
else:
    print('Mean value in mead plane= ', np.mean(mm))



from __future__ import division
from past.utils import old_div

from numpy import save, load, angle
import matplotlib.pyplot as plt

fphi = load('fphi.npy')

fte = load('fte.npy')
phase_te = angle(old_div(fphi,fte))
save('phase_te', phase_te)
plt.figure()
plt.plot(mean(mean(phase_te[:,:,3,:],axis=1),axis=1))
plt.title('Te')
plt.savefig('image/phase_te.png')
plt.savefig('image/phase_te.eps')

fti = load('fti.npy')
phase_ti = angle(old_div(fphi,fti))
save('phase_ti', phase_ti)
plt.figure()
plt.plot(mean(mean(phase_ti[:,:,3,:],axis=1),axis=1))
plt.title('ti')
plt.savefig('image/phase_ti.png')
plt.savefig('image/phase_ti.eps')

fni = load('fni.npy')
phase_ni = angle(old_div(fphi,fni))
save('phase_ni', phase_ni)
plt.figure()
plt.plot(mean(mean(phase_ni[:,:,3,:],axis=1),axis=1))
plt.title('ni')
plt.savefig('image/phase_ni.png')
plt.savefig('image/phase_ni.eps')

plt.show()




#!/usr/bin/env python

import numpy as np
import zoidberg
   
field = zoidberg.field.GEQDSK("g014220.00200") # Read magnetic field

grid = zoidberg.grid.rectangular_grid(100, 10, 100,
                                      1.5-0.1, # Range in R (max - min)
                                      2*np.pi, # Toroidal angle
                                      3., # Range in Z
                                      xcentre=(1.5+0.1)/2, # Middle of grid in R
                                      yperiodic=True) # Periodic in toroidal angle

# Create the forward and backward maps
maps = zoidberg.make_maps(grid, field)

# Save to file
zoidberg.write_maps(grid, field, maps, gridfile="tokamak.fci.nc")

#############################################################################
# Plot maps

zoidberg.plot.plot_forward_map(grid, maps)




from builtins import range
####################################
# computes polslice for time series
####################################

import numpy as np
from boutdata.collect import collect
from boututils.plotpolslice import plotpolslice

###########################
# Specify parameters


path='./data/'

variable="P"

p = collect(variable, path=path)

period=15

grid='../cbm18_dens8.grid_nx68ny64.nc'

########################################################
# Call plotpolslice once to get extended poloidal grid

r,z,fun=plotpolslice(p[0,:,:,:],grid,period=period,rz=1)

nx=r.shape[0] # number of points in r
ny=r.shape[1] # number of points in z
nt=p.shape[0] # time intervals


fm=np.zeros((nt,nx,ny)) # array to store the time sequence of the poloidal cross section

#Compute all time frames

for k in range(nt):
    fm[k,:,:]=plotpolslice(p[k,:,:,:],grid,period=period,rz=0)

np.savez('pslice',fm=fm, z=z, r=r)



#!/usr/bin/env python3
#
# Creates the grid files and input options for
# a set of simulations. These can then be submitted
# to a batch queue and then processed with runtest
# (setting "running = False")
#

from __future__ import print_function
from __future__ import division

nxlist = [4, 8, 16, 32, 64, 128]

from boututils.datafile import DataFile
from boututils.run_wrapper import shell

from os.path import isfile

from boutdata.mms import SimpleTokamak, x, y
from sympy import sin, cos

from math import pi

shape = SimpleTokamak()

## Add equilibrium profiles
MU0 = 4.0e-7 * pi

J0 = 1 - x - sin(x * pi) ** 2 * cos(y)  # Parallel current
P0 = 2 + cos(x * pi)  # Pressure pedestal
bxcvz = -((1.0 / shape.Rxy) ** 2) * cos(y)  # Curvature

# Normalisation
Lbar = 1.0
Bbar = 1.0
J0 = -J0 * shape.Bxy / (MU0 * Lbar)  # Turn into A/m^2
P0 = P0 * Bbar**2 / (2.0 * MU0)  # Pascals

shape.add(P0, "pressure")
shape.add(J0, "Jpar0")
shape.add(bxcvz, "bxcvz")

for nx in nxlist:
    # Generate a new mesh file

    filename = "grid%d.nc" % nx

    if isfile(filename):
        print("Grid file '%s' already exists" % filename)
    else:
        print("Creating grid file '%s'" % filename)
        f = DataFile(filename, create=True)
        shape.write(nx, nx, f)
        f.close()

    # Generate BOUT.inp file

    directory = "grid%d" % nx
    shell("mkdir " + directory)
    shell("cp data/BOUT.inp " + directory)
    shell("sed -i 's/MZ = 17/MZ = %d/g' %s/BOUT.inp" % (nx, directory))
    shell(
        'sed -i \'s/grid = "grid16.nc"/grid = "%s"/g\' %s/BOUT.inp'
        % (filename, directory)
    )



import numpy as np
from scipy import optimize
import matplotlib.pyplot as plt

Te_ref = np.loadtxt("temperature.csv", delimiter=",")
Te_ref[:,0] *= 1e-4 # Convert X axis to m

def te_function(ypos, mid, wwid, w0, w1, w2, Tmax, Tmin, clip=False):
    width = w0 + ((ypos - mid)*w1 + (ypos - mid)**2 * w2) * np.exp(-((ypos - mid)/wwid)**2)

    if clip:
        width = np.clip(width, 1e-10, None)

    return Tmax - 0.5 * (1 + np.tanh((ypos - mid)/width)) * (Tmax - Tmin)

popt, pcov = optimize.curve_fit(te_function, Te_ref[:,0], Te_ref[:,1],
                                p0 = [2.2e-4, 1e-4, 1e-4, 0.0, 0.0, 0.960, 0.190])

print(popt)

xfit = np.linspace(Te_ref[0,0], Te_ref[-1,0], 100)

plt.plot(xfit, te_function(xfit, *popt, clip=True), '-k')
plt.plot(Te_ref[:,0], Te_ref[:,1], 'or')
plt.show()



from __future__ import print_function
from __future__ import division
from builtins import range
from past.utils import old_div
# the Scientific Python netCDF 3 interface
# http://dirac.cnrs-orleans.fr/ScientificPython/
#from Scientific.IO.NetCDF import NetCDFFile as Dataset
# the 'classic' version of the netCDF4 python interface
# http://code.google.com/p/netcdf4-python/
import numpy as np
from netCDF4 import Dataset
from numpy import dtype # array module from http://numpy.scipy.org
"""
This example writes some surface pressure and temperatures
The companion program sfc_pres_temp_rd.py shows how to read the netCDF
data file created by this program.

This example demonstrates the netCDF Python API.
It will work either with the Scientific Python NetCDF version 3 interface
(http://dirac.cnrs-orleans.fr/ScientificPython/)
of the 'classic' version of the netCDF4 interface. 
(http://netcdf4-python.googlecode.com/svn/trunk/docs/netCDF4_classic-module.html)
To switch from one to another, just comment/uncomment the appropriate
import statements at the beginning of this file.

Jeff Whitaker <jeffrey.s.whitaker@noaa.gov> 20070202
"""
# Adapted for BOUT++ by
# George Breyiannis, JAEA, Nov 2013
#

# the output array to write will be nx x ny
ny = 100; nx = ny + 4

# dy of grid
dy = old_div(1.0, np.float(ny))
dx = dy
# create grid
dxarr=np.zeros((nx,ny),dtype='float32')+dx
dyarr=np.zeros((nx,ny),dtype='float32')+dy

xarr=np.arange(0.,np.float(nx),1.,dtype='float32')*dx
yarr=np.arange(0.,np.float(ny),1.,dtype='float32')*dy

# compute initial variables

rho=np.zeros((nx,ny),dtype='float32')+old_div(25.,(36.*np.pi))
p=np.zeros((nx,ny),dtype='float32')+old_div(5.,(12.*np.pi))

rho=1.
p=old_div(rho,3.)

v_x=np.zeros((nx,ny),dtype='float32')
Bx=np.zeros((nx,ny),dtype='float32')

for y in range(ny):
    v_x[:,y]=-np.sin(2.*np.pi*yarr[y])
    Bx[:,y]=-np.sin(2.*np.pi*yarr[y])
 
#Bx=Bx/np.sqrt(4.*np.pi)


v_y=np.zeros((nx,ny),dtype='float32')
By=np.zeros((nx,ny),dtype='float32')

for x in range(nx):
    v_y[x,:]=np.sin(2.*np.pi*xarr[x])
    By[x,:]=np.sin(4.*np.pi*xarr[x])
    
#By=By/np.sqrt(4.*np.pi)


# Domain inside core (periodic)

ixseps1 = nx
ixseps2 = nx

# open a new netCDF file for writing.
ncfile = Dataset('otv.grd.128.nc','w', format='NETCDF3_CLASSIC') 

# output data.

# create the nx and ny dimensions.
ncfile.createDimension('x',nx)
ncfile.createDimension('y',ny)
ncfile.createDimension('single',1)

# create and write nx,ny variables
nxx=ncfile.createVariable('nx','i4',('single'))
nyy=ncfile.createVariable('ny','i4',('single'))

nxx[:]=nx
nyy[:]=ny

# Define the coordinate variables. They will hold the coordinate
# information, that is, xarr,yarr
dx = ncfile.createVariable('dx',dtype('float32').char,('x','y'))
dy = ncfile.createVariable('dy',dtype('float32').char,('x','y',))

# write data to coordinate vars.
dx[:,:] = dxarr
dy[:,:] = dyarr

# create and write ixseps* dimensions.
ix1=ncfile.createVariable('ixseps1','i4',('single'))
ix2=ncfile.createVariable('ixseps2','i4',('single'))

ix1[:]=ixseps1
ix2[:]=ixseps2

# create the corresponding variables 
rho0 = ncfile.createVariable('rho0',dtype('float32').char,('x','y'))
p0 = ncfile.createVariable('p0',dtype('float32').char,('x','y'))
v0_x = ncfile.createVariable('v0_x',dtype('float32').char,('x','y'))
v0_y = ncfile.createVariable('v0_y',dtype('float32').char,('x','y'))
B0x = ncfile.createVariable('B0x',dtype('float32').char,('x','y'))
B0y = ncfile.createVariable('B0y',dtype('float32').char,('x','y'))

# write data to variables.

rho0[:,:]=rho
p0[:,:]=p
v0_x[:,:]=v_x
v0_y[:,:]=v_y
B0x[:,:]=Bx
B0y[:,:]=By


ncfile.close()
print('*** SUCCESS writing file otv.grd.py.nc!')



#!/usr/bin/env python

# Create grids for a straight stellarator, based on curvilinear grids

import numpy as np

import zoidberg


#############################################################################
# Define the magnetic field
    
# Length in y after which the coils return to their starting (R,Z) locations
yperiod = 10.

magnetic_field = zoidberg.field.StraightStellarator(I_coil=0.3, radius = 1.0, yperiod = yperiod)

#############################################################################
# Create the inner flux surface, starting at a point at phi=0
# To do this we need to define the y locations of the poloidal points
# where we will construct grids

start_r = 0.2
start_z = 0.0

nslices = 8  # Number of poloidal slices
ycoords = np.linspace(0, yperiod, nslices)
npoints = 20  # Points per poloidal slice

rzcoord, ycoords = zoidberg.fieldtracer.trace_poincare(magnetic_field, start_r, start_z, yperiod,
                                                     y_slices=ycoords, revs=npoints)
    
inner_lines = []
for i in range(nslices):
    r = rzcoord[:,i,0]
    z = rzcoord[:,i,1]
    line = zoidberg.rzline.line_from_points(r,z)
    # Re-map the points so they're approximately uniform in distance along the surface
    # Note that this results in some motion of the line
    line = line.equallySpaced()
    inner_lines.append(line)
    
# Now have a list of y coordinates (ycoords) and inner lines (inner_lines)

#############################################################################
# Generate a fixed circle for the outer boundary

outer_line = zoidberg.rzline.circle(R0=0.0, r=0.8)

#############################################################################
# Now have inner and outer boundaries for each poloidal grid
# Generate a grid on each poloidal slice using the elliptic grid generator

nx = 20
ny = 20

pol_grids = [ zoidberg.poloidal_grid.grid_elliptic(inner_line, outer_line, nx,ny) for inner_line in inner_lines ]

#############################################################################
# Create a grid, then calculate forward and backward maps

grid = zoidberg.grid.Grid( pol_grids, ycoords, yperiod, yperiodic=True)

maps = zoidberg.make_maps(grid, magnetic_field)

#############################################################################
# Write grid file

filename = "stellarator.fci.nc"

print("Writing to grid file '{0}'".format(filename))
zoidberg.write_maps(grid, magnetic_field, maps, gridfile=filename, new_names=False, metric2d=True)

#############################################################################
# Plot maps

zoidberg.plot.plot_forward_map(grid, maps)



#!/usr/bin/env python

# Create grids for a straight stellarator, based on 
# curvilinear grids

import numpy as np
import matplotlib.pyplot as plt

import zoidberg

#############################################################################
# Define the magnetic field
    
# Length in y after which the coils return to their starting (R,Z) locations
yperiod = 10.

magnetic_field = zoidberg.field.StraightStellarator(I_coil=0.4, radius = 1.0, yperiod = yperiod)

start_r = 0.3
start_z = 0.0

nslices = 4  # Number of poloidal slices
ycoords = np.linspace(0, yperiod, nslices)
npoints = 60  # Points per poloidal slice

# Create a field line tracer
tracer = zoidberg.fieldtracer.FieldTracer(magnetic_field)
#tracer = zoidberg.fieldtracer.FieldTracerReversible(magnetic_field)

# Extend the y coordinates so the tracer loops npoints times around yperiod
ycoords_all = ycoords
for i in range(1,npoints):
    ycoords_all = np.append(ycoords_all, ycoords + i*yperiod)
    
coord = tracer.follow_field_lines(start_r, start_z, ycoords_all, rtol=1e-12)

for i in range(nslices):
    r = coord[i::nslices,0]
    z = coord[i::nslices,1]
    plt.plot(r,z,'o')

#i = 1
#plt.plot(coord[i::nslices,0], coord[i::nslices,1],'o')

plt.show()

    



#!/usr/bin/env python

# Demonstrates how to generate 3D plots of magnetic field lines

import zoidberg

# Size of the domain in y (periodic)
yperiod = 10.

# Define magnetic field
magnetic_field = zoidberg.field.StraightStellarator(I_coil=0.4, radius = 1.0, yperiod = yperiod)

# Make 3D plot
zoidberg.plot.plot_3d_field_line(magnetic_field, 
                                 0.3,                    # x starting position
                                 0.0,                    # z starting position
                                 yperiod,                # Periodicity of y domain
                                 cycles = 20)
